// Matrix routines // RjY/anARCHy

#ifndef MATRIX_H
#define MATRIX_H

typedef unsigned int Dimension;

typedef double Real;

typedef Real* VElts; // vector, i.e. Nx1 array of elements
typedef Real** MElts; // array of arrays of elements

typedef struct Vector_s { // Nx1 column (or 1xN row, whichever)
	Dimension N;
	VElts v;
} Vector;

typedef struct Matrix_s {
	Dimension N; // NxN matrix
	MElts m; // 2d array of elements, size NxN (duh)
} Matrix;

// create a new matrix, initialised to zero
Matrix *matrix_new(Dimension);

// free a matrix
void matrix_free(Matrix *);

// create a new matrix, initialised from supplied array of elements
Matrix *matrix_new_elts(Dimension, MElts);

// duplicate a matrix
Matrix *matrix_dup(Matrix *);

// get element
Real matrix_get_elt(Matrix *, int, int);

// set element to value
void matrix_set_elt(Matrix *, int, int, Real);

// dump the contents of a matrix to stdout (debugging routine)
void matrix_print(Matrix *);

// Adds the second matrix onto the first
Matrix *matrix_add(Matrix *, Matrix *);

// Multiplies the matrix by a scalar
Matrix *matrix_scalarmult(Matrix *, Real);

// Returns the multiple of two NxN matrices
Matrix *matrix_mult(Matrix *, Matrix *);

// transpose a matrix in place. Returns a pointer to the same matrix
Matrix *matrix_transpose(Matrix *);

// Given an Nd matrix return an (N-1)d matrix removing the specified row/column
Matrix *matrix_chomp(Matrix *, int, int);

// Recursive routine to calculate a matrix determinant.
Real matrix_det(Matrix *);

// Find the cofactor matrix of a given matrix (this is expensive)
Matrix *matrix_cofactor(Matrix *);

// Find the inverse of a given matrix
Matrix *matrix_invert(Matrix *);

#endif // MATRIX_H
